@@ -54,7 +54,9 @@ Package::Package(PackageStructure *structure)
 
     if (d->structure) {
         d->structure.data()->initPackage(this);
-        addFileDefinition("metadata", QStringLiteral("metadata.desktop"), i18n("Desktop file that describes this package."));
+        auto desc = i18n("Desktop file that describes this package.");
+        addFileDefinition("metadata", QStringLiteral("metadata.json"), desc);
+        addFileDefinition("metadata", QStringLiteral("metadata.desktop"), desc);
     }
 }
 
@@ -273,7 +275,7 @@ QString PackagePrivate::unpack(const QString &filePath)
             // search metadata.desktop, the zip file might have the package contents in a subdirectory
             QDir unpackedPath(tempdir.path());
             const auto entries = unpackedPath.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
-            foreach (const auto &pack, entries) {
+            for (const auto &pack : entries) {
                 if (QFile::exists(pack.filePath() + QLatin1String("/metadata.json")) || QFile::exists(pack.filePath() + QLatin1String("/metadata.desktop"))) {
                     tempRoot = pack.filePath() + QLatin1Char('/');
                 }
@@ -341,7 +343,7 @@ QString Package::filePath(const QByteArray &fileType, const QString &filename) c
 
     QStringList paths;
 
-    if (fileType.size()) {
+    if (!fileType.isEmpty()) {
         const auto contents = d->contents.constFind(fileType);
         if (contents == d->contents.constEnd()) {
             //qCDebug(KPACKAGE_LOG) << "package does not contain" << fileType << filename;
@@ -362,21 +364,21 @@ QString Package::filePath(const QByteArray &fileType, const QString &filename) c
 
     //Nested loop, but in the medium case resolves to just one iteration
 //     qCDebug(KPACKAGE_LOG) << "prefixes:" << d->contentsPrefixPaths.count() << d->contentsPrefixPaths;
-    foreach (const QString &contentsPrefix, d->contentsPrefixPaths) {
+    for (const QString &contentsPrefix : qAsConst(d->contentsPrefixPaths)) {
         QString prefix;
         //We are an installed package
         if (d->tempRoot.isEmpty()) {
             prefix = fileType == "metadata" ? d->path : (d->path + contentsPrefix);
-        //We are a compressed package temporarly uncompressed in /tmp
+        //We are a compressed package temporarily uncompressed in /tmp
         } else {
             prefix = fileType == "metadata" ? d->tempRoot : (d->tempRoot + contentsPrefix);
         }
 
-        foreach (const QString &path, paths) {
+        for (const QString &path : qAsConst(paths)) {
             QString file = prefix + path;
 
             if (!filename.isEmpty()) {
-                file = file + QLatin1Char('/') + filename;
+                file.append(QLatin1Char('/') + filename);
             }
 
             QFileInfo fi(file);
@@ -427,9 +429,10 @@ QStringList Package::entryList(const QByteArray &key) const
 
     //qCDebug(KPACKAGE_LOG) << "going to list" << key;
     QStringList list;
-    foreach (const QString &prefix, d->contentsPrefixPaths) {
+    for (const QString &prefix : qAsConst(d->contentsPrefixPaths)) {
         //qCDebug(KPACKAGE_LOG) << "     looking in" << prefix;
-        foreach (const QString &path, it.value().paths) {
+        const QStringList paths = it.value().paths;
+        for (const QString &path : paths) {
             //qCDebug(KPACKAGE_LOG) << "         looking in" << path;
             if (it.value().directory) {
                 //qCDebug(KPACKAGE_LOG) << "it's a directory, so trying out" << d->path + prefix + path;
@@ -539,7 +542,7 @@ void Package::setPath(const QString &path)
     // now we search each path found, caching our previous path to know if
     // anything actually really changed
     const QString previousPath = d->path;
-    foreach (const QString &p, paths) {
+    for (const QString &p : qAsConst(paths)) {
         d->checkedValid = false;
         QDir dir(p);
 
@@ -549,7 +552,9 @@ void Package::setPath(const QString &path)
         if (dir.exists(QStringLiteral("contents.rcc"))) {
             d->rccPath = dir.absoluteFilePath(QStringLiteral("contents.rcc"));
             QResource::registerResource(d->rccPath);
-            dir = QDir(QStringLiteral(":/") + defaultPackageRoot() + path);
+
+            //we need just the plugin name here, never the absolute path
+            dir = QDir(QStringLiteral(":/") + defaultPackageRoot() + path.midRef(path.lastIndexOf(QLatin1Char('/'))));
         }
 
         d->path = dir.canonicalPath();
@@ -653,7 +658,7 @@ QByteArray Package::cryptographicHash(QCryptographicHash::Algorithm algorithm) c
         qCWarning(KPACKAGE_LOG) << "no metadata at" << metadataPath;
     }
 
-    foreach (const QString &prefix, d->contentsPrefixPaths) {
+    for (const QString &prefix : qAsConst(d->contentsPrefixPaths)) {
         const QString basePath = d->path + prefix;
         QDir dir(basePath);
 
@@ -935,7 +940,8 @@ void PackagePrivate::updateHash(const QString &basePath, const QString &subPath,
 
     const QDir::SortFlags sorting = QDir::Name | QDir::IgnoreCase;
     const QDir::Filters filters = QDir::Hidden | QDir::System | QDir::NoDotAndDotDot;
-    foreach (const QString &file, dir.entryList(QDir::Files | filters, sorting)) {
+    const auto lstEntries = dir.entryList(QDir::Files | filters, sorting);
+    for (const QString &file : lstEntries) {
         if (!subPath.isEmpty()) {
             hash.addData(subPath.toUtf8());
         }
@@ -958,7 +964,8 @@ void PackagePrivate::updateHash(const QString &basePath, const QString &subPath,
         }
     }
 
-    foreach (const QString &subDirPath, dir.entryList(QDir::Dirs | filters, sorting)) {
+    const auto lstEntries2 = dir.entryList(QDir::Dirs | filters, sorting);
+    for (const QString &subDirPath : lstEntries2) {
         const QString relativePath = subPath + subDirPath + QLatin1Char('/');
         hash.addData(relativePath.toUtf8());
 
